@@ -15,7 +15,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.03';
+$VERSION = '1.07';
 
 # APE metadata blocks
 %Image::ExifTool::APE::Main = (
@@ -32,6 +32,11 @@ $VERSION = '1.03';
     Title   => { },
     Track   => { },
     Year    => { },
+    DURATION => {
+        Name => 'Duration',
+        ValueConv => '$val += 4294967296 if $val < 0 and $val >= -2147483648; $val * 1e-7',
+        PrintConv => 'ConvertDuration($val)',
+    },
     'Tool Version' => { Name => 'ToolVersion' },
     'Tool Name'    => { Name => 'ToolName' },
 );
@@ -72,6 +77,24 @@ $VERSION = '1.03';
     10 => { Name => 'SampleRate',      Format => 'int32u' },
 );
 
+# APE Composite tags
+%Image::ExifTool::APE::Composite = (
+    GROUPS => { 2 => 'Audio' },
+    Duration => {
+        Require => {
+            0 => 'APE:SampleRate',
+            1 => 'APE:TotalFrames',
+            2 => 'APE:BlocksPerFrame',
+            3 => 'APE:FinalFrameBlocks',
+        },
+        RawConv => '($val[0] && $val[1]) ? (($val[1] - 1) * $val[2] + $val[3]) / $val[0]: undef',
+        PrintConv => 'ConvertDuration($val)',
+    },
+);
+
+# add our composite tags
+Image::ExifTool::AddCompositeTags('Image::ExifTool::APE');
+
 #------------------------------------------------------------------------------
 # Make tag info hash for specified tag
 # Inputs: 0) tag name, 1) tag table ref
@@ -83,7 +106,9 @@ sub MakeTag($$)
     # remove invalid characters in tag name and capitalize following letters
     $name =~ s/[^\w-]+(.?)/\U$1/sg;
     $name =~ s/([a-z0-9])_([a-z])/$1\U$2/g;
-    AddTagToTable($tagTablePtr, $tag, { Name => $name });
+    my %tagInfo = ( Name => $name );
+    $tagInfo{Groups} = { 2 => 'Preview' } if $tag =~ /^Cover Art/ and $tag !~ /Desc$/;
+    AddTagToTable($tagTablePtr, $tag, \%tagInfo);
 }
 
 #------------------------------------------------------------------------------
@@ -108,7 +133,7 @@ sub ProcessAPE($$)
 
     # check APE signature and process audio information
     # unless this is some other type of file
-    unless ($$et{VALUE}{FileType}) {
+    unless ($$et{FileType}) {
         $raf->Read($buff, 32) == 32 or return 0;
         $buff =~ /^(MAC |APETAGEX)/ or return 0;
         $et->SetFileType();
@@ -141,7 +166,7 @@ sub ProcessAPE($$)
         # look for the APE trailer footer...
         my $footPos = -32;
         # (...but before the ID3v1 trailer if it exists)
-        $footPos -= 128 if $$et{DoneID3} == 2;
+        $footPos -= $$et{DoneID3} if $$et{DoneID3} > 1;
         $raf->Seek($footPos, 2)     or return 1;
         $raf->Read($buff, 32) == 32 or return 1;
         $buff =~ /^APETAGEX/        or return 1;
@@ -192,7 +217,7 @@ sub ProcessAPE($$)
             $val = \$buf2;
             # extract cover art description separately (hackitty hack)
             if ($tag =~ /^Cover Art/) {
-                $buf2 =~ s/^([\x20-\x7f]*)\0//;
+                $buf2 =~ s/^([\x20-\x7e]*)\0//;
                 if ($1) {
                     my $t = "$tag Desc";
                     my $v = $1;
@@ -238,7 +263,7 @@ Currently doesn't parse MAC header unless it is at the start of the file.
 
 =head1 AUTHOR
 
-Copyright 2003-2015, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2024, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
